AC_DEFUN(KDEV_PATH_QGL,
[
AC_REQUIRE([AC_PATH_QT])
AC_REQUIRE([AC_HAVE_GL])

AC_MSG_CHECKING([for QT-OpenGL])

ac_qgl_includes=NO ac_qgl_libraries=NO
qgl_libraries=""
qgl_includes=""
AC_ARG_WITH(qgl-dir,
    [  --with-qgl-dir=DIR       where the root of QT-OpenGL is installed ],
    [  ac_qgl_includes="$withval"/include
       ac_qgl_libraries="$withval"/lib
    ])

AC_ARG_WITH(qgl-includes,
    [  --with-qgl-includes=DIR  where the QT-OpenGL includes are. ],
    [
       ac_qgl_includes="$withval"
    ])


AC_ARG_WITH(qgl-libraries,
    [  --with-qgl-libraries=DIR where the QT-OpenGL library is installed.],
    [  ac_qgl_libraries="$withval"
    ])

# stop if there is no gl base system
if test "$ac_cv_have_gl" = no; then
  AC_MSG_ERROR([Qt-OpenGL needs MesaGL or GL libraries. Please check your installation! ]);
fi

AC_CACHE_VAL(ac_cv_have_qgl,
[#try to guess QtGL locations

qgl_incdirs="$ac_qt_includes"
if test ! "$ac_qgl_includes" = "NO"; then
  qgl_incdirs="$ac_qgl_includes $qgl_incdirs"
fi
kdev_qgl_header=qgl.h

AC_FIND_FILE($kdev_qgl_header, $qgl_incdirs, qgl_incdir)
ac_qgl_includes="$qgl_incdir"

qgl_libdirs="$ac_qt_libraries"
if test ! "$ac_qgl_libraries" = "NO"; then
  qgl_libdirs="$ac_qgl_libraries $qgl_libdirs"
fi

test=NONE
qgl_libdir=NONE
for dir in $qgl_libdirs; do
  try="ls -1 $dir/libqgl.*"
  if test -n "`$try 2> /dev/null`"; then qgl_libdir=$dir; break; else echo "tried $dir" >&AC_FD_CC ; fi
done

AC_LANG_SAVE
AC_LANG_CPLUSPLUS

ac_cxxflags_safe="$CXXFLAGS"
ac_ldflags_safe="$LDFLAGS"
ac_libs_safe="$LIBS"

CXXFLAGS="$CXXFLAGS $GLINC -I$qgl_incdir $all_includes"
LDFLAGS="-L$qgl_libdir $all_libraries $GL_LDFLAGS $USER_LDFLAGS"
LIBS="$LIBS $LIBQT -lXmu -lqgl"
if test "$ac_cv_have_gl" = "mesa"; then
  LIBS="$LIBS -lMesaGL -lMesaGLU"
else
  LIBS="$LIBS -lGL -lGLU"
fi

cat > conftest.$ac_ext <<EOF
#include <qgl.h>
#include <stdio.h>
int main()
{
printf("ac_qgl_libraries=");
printf( (QGLFormat::hasOpenGL()) ? "\"$qgl_libdir\"" : "\"NO\"");
return 0;
}
EOF

if AC_TRY_EVAL(ac_link) && test -s conftest ; then
  eval `./conftest 2>/dev/null`
  rm -f conftest*
else
  echo "configure: failed program was:" >&AC_FD_CC
  cat conftest.$ac_ext >&AC_FD_CC
  ac_qgl_libraries="NO"
fi
rm -f conftest*
CXXFLAGS="$ac_cxxflags_safe"
LDFLAGS="$ac_ldflags_safe"
LIBS="$ac_libs_safe"

AC_LANG_RESTORE
if test "$ac_qgl_includes" = NO || test "$ac_qgl_libraries" = NO; then
  ac_cv_have_qgl="have_qgl=no"
  ac_qgl_notfound=""
  if test "$ac_qgl_includes" = NO; then
    if test "$ac_qgl_libraries" = NO; then
      ac_qgl_notfound="(headers and libraries)";
    else
      ac_qgl_notfound="(headers)";
    fi
  else
    ac_qgl_notfound="(libraries)";
  fi

  AC_MSG_ERROR([Qt-OpenGL $ac_qgl_notfound not found. Please check your installation! ]);
else
  have_qgl="yes"
fi
])

eval "$ac_cv_have_qgl"

if test "$have_qgl" != yes; then
  AC_MSG_RESULT([$have_qgl]);
else
  ac_cv_have_qgl="have_qgl=yes \
    ac_qgl_includes=$ac_qgl_includes ac_qgl_libraries=$ac_qgl_libraries"
  AC_MSG_RESULT([libraries $ac_qgl_libraries, headers $ac_qgl_includes])

  qgl_libraries="$ac_qgl_libraries"
  qgl_includes="$ac_qgl_includes"
fi

AC_SUBST(qgl_libraries)
AC_SUBST(qgl_includes)

if test "$GLINC" != "" && test "$have_qgl" = yes; then
  all_includes="$all_includes $GLINC"
fi
if test "$qgl_includes" = "$qt_includes" || test -z "$qgl_includes"; then
 QGL_INCLUDES="";
else
 QGL_INCLUDES="-I$qgl_includes"
 all_includes="$all_includes $QGL_INCLUDES"
fi

if test "$GL_LDFLAGS" != "" && test "$have_qgl" = yes; then
  all_libraries="$all_libraries $GL_LDFLAGS"
fi
if test "$qgl_libraries" = "$x_libraries" || test "$qgl_libraries" = "$qt_libraries" || test -z "$qgl_libraries"; then
 QGL_LDFLAGS=""
else
 QGL_LDFLAGS="-L$qgl_libraries"
 all_libraries="$all_libraries $QGL_LDFLAGS"
fi

AC_SUBST(QGL_INCLUDES)
AC_SUBST(QGL_LDFLAGS)

if test "$ac_cv_have_gl" = "mesa"; then
  LIB_QGL="-lXmu -lqgl -lMesaGL -lMesaGLU"
else
  LIB_QGL="-lXmu -lqgl -lGL -lGLU"
fi
AC_SUBST(LIB_QGL)

])

